# -*- coding: utf-8 -*-

"""
Wrath of the Lich King (WotLK) expansion module.
"""

from .mapping import Client
from .mapping import AccLvlMapping
from .mapping import CharLvlMapping
from .mapping import WtfMapping
from .dataset import to_module
